<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Command
 */
abstract class Hg_Command {
	protected $service;
	protected $status;
	protected $output;
	protected $options         = array();
	protected $validOptions    = array();
	protected $optionalOptions = array();
	protected $requiredOptions = array();
	protected $globalOptions   = array(
		'encoding' => null,
		'quiet'    => null,
		'verbose'  => null,
		'debug'    => null,
		'include'  => null,
		'exclude'  => null,
		'cwd'      => null,
		'format'   => null
	);

	abstract function __construct($params, Hg_Service $service);

	public function __call($method, $arguments) {
		switch ($method) {
			case 'run':
				return $this->execute($arguments);
			break;
			default:
				if (method_exists($this, $method)) {
					return call_user_func_array(array($this, $method), $arguments);
				} else {
					throw new Hg_Exception("The method '{$method}' does not exist in this class.");
				}
		}
	}

	public function setOptions(array $options = array()) {
		if (count($options) > 0) {
			$options = $options[0];

			if (is_array($options)) {
				foreach ($options as $key => $value) {
					if (is_numeric($key)) {
						unset($options[$key]);
						$options[$value] = null;
					}
						else {
							$options[$key] = $value;
						}
				}

				$this->addOptions($options);

			} else if (is_string($options)) {
				$this->addOption($options, null);
			}
		}
	}

	protected function addOption($name, $value = null) {
		if (empty($this->validOptions)) {
			$this->validOptions = array_merge(
				$this->allowedOptions,
				$this->globalOptions,
				$this->requiredOptions
			);
		}

		if (!array_key_exists($name, $this->validOptions)) {
			throw new Hg_Exception("The option '{$name}' is not an valid option");
		}

		if (is_object($value)) {
			$type  = get_class($value);
			$value = print_r($type, true);
		}

		$this->options[$name] = $value;

		return true;
	}

	protected function addOptions(array $options) {
		if (!is_array($options)) {
			throw new Hg_Exception('Options is not an array');
		}

		foreach ($options as $name => $value) {
			$this->addOption($name, $value);
		}

		return true;
	}

	public function getOption($name) {
		$value = null;

		if (isset($this->options[$name])) {
			$value = $this->options[$name];
		}

		return $value;
	}

	public function getOptions() {
		return $this->options;
	}

	protected function formatOptions(array $options) {
		$modifiers      = null;
		$missingOptions = array_diff_key($this->requiredOptions, $options);

		if (count($missingOptions) > 0) {
			throw new Hg_Exception("Required option(s) are missing: " . implode(', ', $missingOptions));
		}

		if (array_key_exists('files', $options)) {
			$files = $options['files'];

			unset($options['files']);

			if (is_array($files)) {
				$files = implode(' ', $files);
			}

			$options[null] = $files;
		}

		if (array_key_exists('from', $options)) {
			unset($options['from']);
		}

		foreach ($options as $option => $argument) {
			$modifiers .= (empty($option)) ? '' : " --{$option}";
			$modifiers .= (empty($argument)) ? '' : ' ' . $argument;
		}

		return $modifiers;
	}

	public function setCommandString() {
		$this->commandString = '"' .
		$this->service->executable . '" ' .
		$this->command . rtrim($this->formatOptions($this->getOptions())) . ' 2>&1';
	}
}
