<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Hg_Repository
 */
class Hg_Repository {
	/**
	 * Hg Directory
	 *
	 * @const string
	 */
	const HG_DIRECTORY = '.hg';

	/**
	 * Repository Path
	 *
	 * @var string
	 */
	protected $path = null;

	/**
	 * Repository service
	 *
	 * @var <Hg_Service
	 */
	protected $service;

	/**
	 * Local hash
	 *
	 * @var <string>
	 */
	protected $localHash = null;

	/**
	 * Remote hash
	 *
	 * @var <string>
	 */
	protected $remoteHash = null;

	/**
	 * Constructor
	 *
	 * @param Hg_Service $service instance of our core class
	 * @param string     $path    path to our repository
	 */
	public function __construct(Hg_Service $service, $path) {
		$this->service = $service;
		$this->setPath($path);
	}

	/**
	 * Reset our repository instance
	 */
	public static function reset() {
		self::$instance = null;
	}

	/**
	 * Checks for a valid mercurial repository
	 *
	 * @return boolean returns true if we have a valid mercurial repository
	 */
	public function isRepository() {
		$path      = $this->getRepositoryHgFolder();
		$directory = new Hg_Util_Directory($path);

		return $directory->exists();
	}

	/**
	 * Set the path to our repository
	 *
	 * @param  string        $path the path to our repository
	 * @return Hg_Repository $this returns the own instance
	 */
	public function setPath($path = null) {
		$this->path = $path;

		return $this;
	}

	/**
	 * Get the path to our repository
	 *
	 * @return string $path returns the path to our repository
	 */
	public function getPath() {
		return $this->path;
	}

	/**
	 * Get the hg folder inside our repository
	 *
	 * @return string $path returns the path to the hg folder inside our repository
	 */
	protected function getRepositoryHgFolder() {
		return $this->path . DIRECTORY_SEPARATOR . self::HG_DIRECTORY;
	}

	/**
	 * Delete a repository
	 */
	public function delete() {
		if (!$this->isRepository()) {
			throw new Hg_Exception('Invalid repository given. Mercurial could not open the repository. Is there an .hg directory or is it corrupted?');
		}

		$directory = new Hg_Util_Directory($this->path);
		$directory->delete(true);

		return $this;
	}

	/**
	 * Get repository config
	 *
	 * @return <string> Repository config
	 */
	public function getConfig() {
		return file_get_contents($this->getRepositoryHgFolder() . DIRECTORY_SEPARATOR . 'hgrc');
	}

	/**
	 * Set repository config
	 *
	 * @param <string> $config Repository config
	 */
	public function setConfig($config) {
		file_put_contents($this->getRepositoryHgFolder() . DIRECTORY_SEPARATOR . 'hgrc', $config, LOCK_EX);
	}

	/**
	 * __toString Method
	 *
	 * @return string $path the path to our repository
	 */
	public function __toString() {
		return $this->getPath();
	}
}
