<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Mercurial cache class
 */
class Mercurial_Cache {
	/**
	 * Cache
	 *
	 * @var <BabelCache_Interface>
	 */
	protected static $cache = null;

	/**
	 * Root namespace
	 *
	 * @var <const>
	 */
	const ROOT_NAMESPACE = 'mercurial';

	/**
	 * Get Cache
	 *
	 * @return <BabelCache_Interface> BabelCache interface instance
	 */
	public static function getCache() {
		if (!self::$cache) self::$cache = sly_Core::cache();
		return self::$cache;
	}

	/**
	 * Get cache value
	 *
	 * @param <mixed> $namespace Extended namespace
	 * @param <mixed> $key Unique cache key
	 * @param <mixed> $default Default value
	 *
	 * @return <mixed> Cached value
	 */
	public static function get($namespace, $key, $default = null) {
		$def  = array('payload' => $default, 'expiry' => 0);
		$data = self::getCache()->get(self::getNamespace($namespace), $key, $def);

		extract($data);

		if ($expiry > 0 && time() > $expiry) {
			self::delete($namespace, $key);
			return $default;
		}

		return $payload;
	}

	/**
	 * Set cache value
	 *
	 * @param <mixed> $namespace Extended namespace
	 * @param <mixed> $key Unique cache key
	 * @param <mixed> $value Valie to set
	 *
	 * @return <boolean> Run of events
	 */
	public static function set($namespace, $key, $value) {
		static $ttl = null;

		if ($ttl === null) {
			$ttl = 86400;
		}

		$expiry = $ttl > 0 ? (time() + $ttl) : 0;
		$data   = array('payload' => $value, 'expiry' => $expiry);

		return self::getCache()->set(self::getNamespace($namespace), $key, $data);
	}

	/**
	 * Delete cache value
	 *
	 * @param <mixed> $namespace Extended namespace
	 * @param <mixed> $key Unique cache key
	 *
	 * @return <boolean> Run of events
	 */
	public static function delete($namespace, $key) {
		return self::getCache()->delete(self::getNamespace($namespace), $key);
	}

	/**
	 * Flush cache value
	 *
	 * @param <mixed> $namespace Extended namespace
	 * @param <mixed> $recursive Recursive cache flushing
	 *
	 * @return <boolean> Run of events
	 */
	public static function flush($namespace = '', $recursive = true) {
		return self::getCache()->flush(self::getNamespace($namespace), $recursive);
	}

	/**
	 * Get namespace
	 *
	 * @param <mixed> $namespace Extended namespace
	 *
	 * @return <string> Generated namespace
	 */
	public static function getNamespace($namespace) {
		return empty($namespace) ? Mercurial_Cache::ROOT_NAMESPACE : Mercurial_Cache::ROOT_NAMESPACE . '.' . $namespace;
	}

	/**
	 * Set cache value based on a flag
	 *
	 * @param <mixed> $flag Flag to match to set cache value
	 * @param <mixed> $namespace Extended namespace
	 * @param <mixed> $key Unique cache key
	 * @param <mixed> $value Valie to set
	 *
	 * @return <boolean> Run of events
	 */
	public static function setIf($flag, $namespace, $key, $value) {
		if ($flag) self::set($namespace, $key, $value);
		else self::remove($namespace, $key);
	}

	/**
	 * Clear cache
	 *
	 * @param <array> $param Parameters to pass
	 *
	 * @return <string> Parameters subject
	 */
	public static function clear(array $params) {
		self::flush();
		return $params['subject'];
	}
}