<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Mercurial_Model_Repository
 */
class Mercurial_Model_Repository implements Mercurial_Interface_Repository {
	/**
	 * Singleton Instance
	 */
	protected static $instances = array();

	/**
	 * Hg service
	 *
	 * @var <Mercurial_Service_Hg>
	 */
	protected $hgService = null;

	/**
	 * Repository service
	 *
	 * @var <Mercurial_Service_Repository>
	 */
	protected $repositoryService = null;

	/**
	 * Repository ID
	 *
	 * @var <int>
	 */
	protected $repositoryID;

	/**
	 * Repository Title
	 *
	 * @var <string>
	 */
	protected $title;

	/**
	 * Repository Path
	 *
	 * @var <string>
	 */
	protected $directory;

	/**
	 * Repository URL Paths
	 *
	 * @var <array>
	 */
	protected $paths = array();

	/**
	 * Repository log
	 *
	 * @var <array>
	 */
	protected $log = array();

	/**
	 * Repository summary
	 *
	 * @var <array>
	 */
	protected $summary = array();

	/**
	 * Singleton Instance
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <Mercurial_Model_Repository> Instance of a repository
	 */
	public static function getInstance($repositoryID) {
		$repositoryID = (int) $repositoryID;

		if (empty(self::$instances[$repositoryID])) {
			self::$instances[$repositoryID] = new self($repositoryID);
		}

		return self::$instances[$repositoryID];
	}

	/**
	 * Public getters
	 */
	public function getID()           { return $this->repositoryID; }
	public function getRepositoryID() { return $this->repositoryID; }
	public function getTitle()        { return $this->title; }
	public function getDirectory()    { return $this->directory; }

	/**
	 * Constructor
	 *
	 * @param <int> $repositoryID Repository ID
	 */
	private function __construct($repositoryID) {
		$this->repositoryID = (int) $repositoryID;

		$database = sly_DB_Persistence::getInstance();
		$prefix   = sly_Core::config()->get('DATABASE/TABLE_PREFIX');

		$database->query(
			sprintf('
				SELECT repository_id, title, directory
				FROM   %smercurial_repository
				WHERE  repository_id = %d'
			, $prefix, $this->repositoryID)
		);

		if (!$database->affectedRows()) {
			throw new Exception(sprintf(t('mercurial_repository_not_found'), $repositoryID));
		}

		foreach ($database as $repository) {
			$this->title     = $repository['title'];
			$this->directory = $repository['directory'];
		}
	}

	/**
	 * Get repository current summary
	 *
	 * @param <string> $key Get single summary value by key
	 *
	 * @return <array> Repository current summary
	 */
	public function getCurrentSummary($key = '') {
		if (empty($this->summary)) {
			$this->summary = $this->getSummary(array());
		}

		return empty($key) ? $this->summary : $this->summary[$key];
	}

	/**
	 * Get repository current log
	 *
	 * @return <string> Repository current log
	 */
	public function getCurrentLog() {
		if (empty($this->log)) {
			$this->log = $this->getLog(array('rev' => $this->getCurrentChangeset()));
		}

		return $this->log;
	}

	/**
	 * Get repository current user
	 *
	 * @return <string> Repository current user
	 */
	public function getCurrentUser() {
		$log = $this->getCurrentLog();

		foreach ($log as $line) {
			if (preg_match('#^user:\s+(.*?)$#i', $line, $match)) {
				$user = $match[1];
			}
		}

		return $user;
	}

	/**
	 * Get repository current date
	 *
	 * @param <string> $format Date format
	 *
	 * @return <string> Repository current date
	 */
	public function getCurrentDate($format = 'd.m.Y') {
		$log = $this->getCurrentLog();

		foreach ($log as $line) {
			if (preg_match('#^date:\s+(.*?)$#i', $line, $match)) {
				$date = date($format, strtotime($match[1]));
			}
		}

		return $date;
	}

	/**
	 * Get repository current commit message
	 *
	 * @return <string> Repository commit message
	 */
	public function getCurrentCommitMessage() {
		$log = $this->getCurrentLog();

		foreach ($log as $line) {
			if (preg_match('#^summary:\s+(.*?)$#i', $line, $match)) {
				$summary = $match[1];
			}
		}

		return $summary;
	}

	/**
	 * Get repository current changeset
	 *
	 * @return <string> Repository current changeset
	 */
	public function getCurrentChangeset() {
		return $this->getCurrentSummary('changeset');
	}

	/**
	 * Get repository current tags
	 *
	 * @return <array> Repository current tags
	 */
	public function getCurrentTags() {
		$tags = $this->getTags();

		return array_key_exists($this->getCurrentChangeset(), $tags) ? $tags[$this->getCurrentChangeset()] : array('tip');
	}

	/**
	 * Get repository subsequent
	 *
	 * @return <int> Repository subsequent
	 */
	public function getCurrentSubsequent() {
		return count($this->getLog(array('rev' => $this->getCurrentChangeset() . '..'), '{node}\n'));
	}

	/**
	 * Get Hg_Service
	 *
	 * @return <Mercurial_Service_Hg> Mercurial_Service_Hg
	 */
	public function getHgService() {
		if ($this->hgService === null) {
			$this->hgService = Mercurial_Service::getService('hg', $this->getDirectory());
		}

		return $this->hgService;
	}

	/**
	 * Get repository service
	 *
	 * @return <Mercurial_Service_Repository> Mercurial_Service_Repository
	 */
	public function getRepositoryService() {
		if ($this->repositoryService === null) {
			$this->repositoryService = Mercurial_Service::getService('repository', $this);
		}

		return $this->repositoryService;
	}

	/**
	 * Pull repository
	 *
	 * @param <array> $options Command options
	 * @param <string> $source Source to pull from
	 *
	 * @return <Hg_Command_Pull> Command instance
	 */
	public function setPull(array $options = array(), $source) {
		return $this->getHgService()->setPull($options, $source);
	}

	/**
	 * Update repository
	 *
	 * @param <array> $options Command options
	 *
	 * @return <Hg_Command_Update> Command instance
	 */
	public function setUpdate(array $options = array()) {
		return $this->getHgService()->setUpdate($options);
	}

	/**
	 * Get repository paths
	 *
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <array> Repository paths
	 */
	public function getPaths($outputType = 'internal', $outputFormat = '') {
		return $this->getHgService()->getPaths($outputType, $outputFormat);
	}

	/**
	 * Get repository incoming
	 *
	 * @param <array> $options Command options
	 * @param <string> $source Source to pull from
	 * @param <string> $template Mercurial incoming template
	 *
	 * @return <Hg_Command_Incoming> Repository incoming
	 */
	public function getIncoming(array $options = array(), $source = 'default', $template = '') {
		return $this->getHgService()->getIncoming($options, $source, $template);
	}

	/**
	 * Get repository outgoing
	 *
	 * @param <array> $options Command options
	 * @param <string> $template Mercurial outgoing template
	 *
	 * @return <Hg_Command_Outgoing> Repository outgoing
	 */
	public function getOutgoing(array $options = array(), $template = '') {
		return $this->getHgService()->getOutgoing($options, $template);
	}

	/**
	 * Get repository status
	 *
	 * @param <array> $options Mercurial status options
	 *
	 * @return <Hg_Command_Status> Repository status
	 */
	public function getStatus(array $options = array()) {
		return $this->getHgService()->getStatus($options);
	}

	/**
	 * Get repository identify
	 *
	 * @param <array> $options Mercurial status options
	 * @param <boolean> $tags Mercurial tags option
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <Hg_Command_Identify> Repository identify
	 */
	public function getIdentify(array $options = array(), $tags = false, $outputType = 'internal', $outputFormat = '') {
		return $this->getHgService()->getIdentify($options, $tags, $outputType, $outputFormat);
	}

	/**
	 * Get repository log
	 *
	 * @param <array> $options Mercurial status options
	 * @param <string> $template Mercurial log tempalte
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <Hg_Command_Log> Repository log
	 */
	public function getLog(array $options = array(), $template = '', $outputType = 'internal', $outputFormat = '') {
		return $this->getHgService()->getLog($options, $template, $outputType, $outputFormat);
	}

	/**
	 * Get repository summary
	 *
	 * @param <array> $options Mercurial status options
	 * @param <boolean> $remote Summary remote
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <Hg_Command_Summary> Repository summary
	 */
	public function getSummary(array $options = array(), $remote = false, $outputType = 'internal', $outputFormat = '') {
		return $this->getHgService()->getSummary($options, $remote, $outputType, $outputFormat);
	}

	/**
	 * Get repository tags
	 *
	 * @param <boolean> $extended Extended tags for repository
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <array> Repository tags
	 */
	public function getTags($extended = false, $outputType = 'internal', $outputFormat = '') {
		return $this->getHgService()->getTags($extended, $outputType, $outputFormat);
	}

	/**
	 * Get repository changesets
	 *
	 * @return <array> Repository changesets
	 */
	public function getChangesets() {
		return $this->getHgService()->getChangesets();
	}

	/**
	 * Get repository config
	 *
	 * @return <string> Repository config values
	 */
	public function getConfig() {
		return $this->getHgService()->getConfig();
	}

	/**
	 * Set repository config
	 *
	 * @param <string> $config Repository config values
	 */
	public function setConfig($config) {
		$this->getHgService()->setConfig($config);
	}

	/**
	 * Edit repository
	 *
	 * @param <array> $data Data to edit the repository to
	 */
	public function update($data) {
		$this->getRepositoryService()->update($data);
	}

	/**
	 * Delete repository
	 */
	public function delete($deleteFolder = false) {
		$this->getRepositoryService()->delete($deleteFolder);
	}
}