<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Mercurial service hg class
 */
class Mercurial_Service_Hg extends Mercurial_Service {
	/**
	 * Hg Service
	 *
	 * @var <Hg_Service>
	 */
	protected $hg = null;

	/**
	 * Constructor
	 *
	 * @param <string> $repository Repository
	 */
	public function __construct($repository = null) {
		$this->hg = Hg::getService($repository);
	}

	/**
	 * Clone repository
	 *
	 * @param <Mercurial_Model_Repository_Container> $container Repository container
	 *
	 * @return <Mercurial_Command_Clone> Instance of clone command
	 */
	public function setClone($container) {
		return $this->hg->clone($container->getSource())->to($container->getDirectory())->run($container->getOptions());
	}

	/**
	 * Add repository
	 *
	 * @param <Mercurial_Model_Repository_Container> $container Repository container
	 *
	 * @throws <Hg_Exception> Thrown Exception when unable to perform action
	 */
	public function setAdd($container) {
		if ($this->hg->repository->isRepository() === false) {
			throw new Hg_Exception(t('mercurial_repository_add_error_message_no_repository'));
		}
	}

	/**
	 * Set repository pull
	 *
	 * @param <array> $options Mercurial update options
	 * @param <string> $source Repository incoming source
	 *
	 * @return <Hg_Command_Pull> Repository pull
	 */
	public function setPull(array $options = array(), $source) {
		return $this->hg->pull($source)->run($options);
	}

	/**
	 * Set repository update
	 *
	 * @param <string> $options Mercurial update options
	 *
	 * @return <Hg_Command_Update> Repository update
	 */
	public function setUpdate(array $options = array()) {
		return $this->hg->update()->run($options);
	}

	/**
	 * Get repository incoming
	 *
	 * @param <array> $options Repository incoming options
	 * @param <string> $source Repository incoming source
	 * @param <string> $template Repository incoming template
	 *
	 * @return <Hg_Command_Incoming> Instance of incoming command
	 */
	public function getIncoming(array $options = array(), $source = 'default', $template = '') {
		return $this->hg->incoming($source)->template($template)->run($options);
	}

	/**
	 * Get repository outgoing
	 *
	 * @param <array> $options Repository outgoing options
	 * @param <string> $template Repository outgoing template
	 *
	 * @return <Hg_Command_Outgoing> Instance of outgoing command
	 */
	public function getOutgoing(array $options = array(), $template = '') {
		return $this->hg->outgoing()->template($template)->run($options);
	}

	/**
	 * Get repository status
	 *
	 * @param <array> $options Mercurial status options
	 *
	 * @return <Hg_Command_Status> Repository status
	 */
	public function getStatus(array $options = array()) {
		return $this->hg->status($options)->run();
	}

	/**
	 * Get repository identify
	 *
	 * @param <array> $options Mercurial status options
	 * @param <boolean> $tags Mercurial tags option
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <Hg_Command_Identify> Repository identify
	 */
	public function getIdentify(array $options = array(), $tags = false, $outputType = 'internal', $outputFormat = '') {
		return $this->hg->identify()->tags($tags)->run($options)->getOutput($outputType, $outputFormat);
	}

	/**
	 * Get repository log
	 *
	 * @param <array> $options Mercurial status options
	 * @param <string> $template Mercurial log tempalte
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <Hg_Command_Log> Repository log
	 */
	public function getLog(array $options = array(), $template = '', $outputType = 'internal', $outputFormat = '') {
		return $this->hg->log()->template($template)->run($options)->getOutput($outputType, $outputFormat);
	}

	/**
	 * Get repository summary
	 *
	 * @param <array> $options Mercurial status options
	 * @param <boolean> $remote Summary remote
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <Hg_Command_Summary> Repository summary
	 */
	public function getSummary(array $options = array(), $remote = false, $outputType = 'internal', $outputFormat = '') {
		return $this->hg->summary()->remote($remote)->run($options)->getOutput($outputType, $outputFormat);
	}

	/**
	 * Get repository paths
	 *
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <Hg_Command_Paths> Repository paths
	 */
	public function getPaths($outputType = 'internal', $outputFormat = '') {
		return $this->hg->paths()->run()->getOutput($outputType, $outputFormat);
	}
	/**
	 * Get repository tags
	 *
	 * @param <boolean> $extended Extended tags for repository
	 * @param <string> $outputType Output type
	 * @param <string> $outputFormat Output format
	 *
	 * @return <array> Repository tags
	 */
	public function getTags($extended = false, $outputType = 'internal', $outputFormat = '') {
		return $this->hg->tags()->extended($extended)->run()->getOutput($outputType, $outputFormat);
	}

	/**
	 * Get repository changesets
	 *
	 * @return <array> Repository changesets
	 */
	public function getChangesets() {
		$tags       = $this->getTags();
		$log        = $this->getLog(array(), '{node|short} {rev}\n');
		$changesets = array();

		foreach ($log as $line) {
			if (preg_match('#^([0-9a-f]+)\s(\d+)#i', $line, $match)) {
				$changeset = $match[1];
				$revision  = $match[2];

				$changesets[$changeset]['revision'] = $revision;

				if (array_key_exists($changeset, $tags)) {
					$changesets[$changeset]['tags'] = $tags[$changeset];
				}
			}
		}

		return $changesets;
	}

	/**
	 * Get repository config
	 *
	 * @return <string> Repository config values
	 */
	public function getConfig() {
		return $this->hg->repository->getConfig();
	}

	/**
	 * Set repository config
	 *
	 * @param <string> $config Repository config values
	 */
	public function setConfig($config) {
		$this->hg->repository->setConfig($config);
	}

	/**
	 * Delete repository folder
	 *
	 * @return <Hg_Repository> Instance of Hg_Repository
	 */
	public function deleteFolder() {
		return $this->hg->repository->delete();
	}
}
