<?php
/*
 * Copyright (c) 2012, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Mercurial controller ajax
 */
class sly_Controller_Mercurial_Ajax extends sly_Controller_Mercurial implements sly_Controller_Interface {
	protected $init;

	/**
	 * Init
	 */
	protected function init() {
		if ($this->init) return;
                $this->init = true;
		$this->renderLayout();
	}

	/**
	 * Get mercurial tree
	 */
	public function getMercurialTreeAction() {
		$this->init();

		$directory   = new Mercurial_Util_Directory();
		$directories = $directory->getBaseDirectories();

		$this->printData($directories);
	}

	/**
	 * Get repository info
	 */
	public function getRepositoryInfoAction() {
		$this->init();

		$repositoryID = sly_request('repositoryID', 'int', 0);
		$forceCache   = sly_request('forceCache', 'boolean', false);
		$cache        = Mercurial_Cache::getCache();
		$namespace    = 'dashboard.info';
		$key          = 'repository.' . $repositoryID;

		if ($forceCache) {
			$cache->delete($namespace, $key);
		}

		$info = $cache->get($namespace, $key);

		if (empty($info)) {
			$info['tags']     = $this->getRepositoryTags($repositoryID);
			$info['incoming'] = $this->getRepositoryIncoming($repositoryID);
			$info['outgoing'] = $this->getRepositoryOutgoing($repositoryID);
			$info['status']   = $this->getRepositoryStatus($repositoryID);
			$info['summary']  = $this->getRepositorySummary($repositoryID);

			$cache->set($namespace, $key, $info);
		}

		$this->printData($info);
	}

	/**
	 * Get current tag
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <array> Tag infos
	 */
	protected function getRepositoryTags($repositoryID) {
		$repository = Mercurial_Provider::getRepository($repositoryID);
		$changeset  = $repository->getCurrentChangeset();
		$data       = $repository->getCurrentTags();
		$tags       = array();

		foreach ($data as $tag) {
			if (empty($tag) || $tag === 'tip') {
				$tags['changeset'] = $changeset;
			}
			else {
				$tags['tags'][] = $tag;
			}

			$tags['subsequent'] = $repository->getCurrentSubsequent();
		}

		return $tags;
	}

	/**
	 * Get repository incoming
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <int> Count of incoming changes
	 */
	protected function getRepositoryIncoming($repositoryID) {
		$repository = Mercurial_Provider::getRepository($repositoryID);
		$data       = $repository->getIncoming(array(), 'default', '{rev}')->getOutput();
		$incoming   = array();

		foreach ($data as $line) {
			if (preg_match('#^(\d+)$#i', $line, $match)) {
				$incoming[] = $match[1];
			}
		}

		return count($incoming);
	}

	/**
	 * Get repository outgoing
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <int> Count of outgoing changes
	 */
	protected function getRepositoryOutgoing($repositoryID) {
		$repository = Mercurial_Provider::getRepository($repositoryID);
		$data       = $repository->getOutgoing(array(), '{rev}')->getOutput();
		$outgoing   = array();

		foreach ($data as $line) {
			if (preg_match('#^(\d+)$#i', $line, $match)) {
				$outgoing[] = $match[1];
			}
		}

		return count($outgoing);
	}

	/**
	 * Get repository file status
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <int> Count of status changes
	 */
	protected function getRepositoryStatus($repositoryID) {
		static $cases = array(
			'modified',
			'added',
			'removed',
			'deleted',
			'unknown'
		);

		$repository = Mercurial_Provider::getRepository($repositoryID);
		$status     = array();

		foreach ($cases as $case) {
			$status[] = count($repository->getStatus(array($case))->getOutput());
		}

		return array_sum(array_filter($status));
	}

	/**
	 * Get repository summary
	 *
	 * @param <int> $repositoryID Repository ID
	 *
	 * @return <int> Repository summary
	 */
	protected function getRepositorySummary($repositoryID) {
		$repository = Mercurial_Provider::getRepository($repositoryID);

		$summary['user']    = $repository->getCurrentUser();
		$summary['date']    = $repository->getCurrentDate();
		$summary['summary'] = $repository->getCurrentCommitMessage();

		return $summary;
	}

	/**
	 * Print JSON Data
	 *
	 * @param <mixed> $data Data to print to JSON
	 */
	protected function printData($data) {
		print json_encode($data);
		die;
	}

	/**
	 * Check permission
	 *
	 * @param <string> $action Action parameter
	 *
	 * @return <boolean> State if user has permission to access this controller
	 */
	public function checkPermission($action) {
		$user = sly_Util_User::getCurrentUser();
		return $user->isAdmin() || $user->hasRight('mercurial[]');
	}

	/**
	 * Render layout
	 */
	protected function renderLayout() {
		while (ob_get_level()) ob_end_clean();
	}
}